using System;
using System.Collections;
using System.Text.RegularExpressions;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileNameMatcher"/> concrete implementation that matches file names with
    /// one or more regular expression.
    ///
    /// <configuration>
    /// This class can be constructed via reflection by the constructor contract of
    /// <see cref="IFileMatcher"/>.
    /// The supported property is "patterns" which is potentially multi-valued.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class RegexMatcher : FileNameMatcher
    {
        /// <summary>
        /// Configuration key that contains multiple regular expressions to match.
        /// The regex are compiled when loaded and invalid pattern will result
        /// in a <see cref="ConfigurationException"/>.
        /// </summary>
        public static readonly string PatternsPropertyKey = "patterns";

        /// <summary>
        /// A list of compiled regular expressionss (Regex) to match.
        /// This list is initialized in place and is immutable.
        /// </summary>
        private IList patterns = new ArrayList();

        /// <summary>
        /// Constructor with single pattern.
        /// </summary>
        ///
        /// <param name="pattern">the regex pattern</param>
        ///
        /// <exception cref="ArgumentNullException">if pattern is null</exception>
        public RegexMatcher(Regex pattern)
        {
            // verify the pattern is valid
            FileStatisticsHelper.CheckNull(pattern, "pattern");

            // add the pattern to patterns
            this.patterns.Add(pattern);
        }

        /// <summary>
        /// Constructor with multiple regular expression patterns.
        /// </summary>
        ///
        /// <param name="patterns">the patterns to match</param>
        ///
        /// <exception cref="ArgumentNullException">if patterns is null</exception>
        /// <exception cref="ArgumentException">
        /// if patterns is empty or contains non-Regex instance</exception>
        public RegexMatcher(IList patterns)
        {
            // verify the patterns is valid
            FileStatisticsHelper.CheckEmptyList(patterns, "patterns");

             //  save a clone of the list
            foreach (object obj in patterns)
            {
                // verify the obj is valid
                if (obj == null)
                {
                    throw new ArgumentException("The patterns contains null.", "patterns");
                }
                if (!(obj is Regex))
                {
                    throw new ArgumentException("The patterns contains non-Regex instance.", "patterns");
                }

                this.patterns.Add(obj);
            }
        }

        /// <summary>
        /// Constructor that loads patterns from a property.
        /// It should look for a "patterns" sub-property and extract multiple values.
        /// Each pattern is compiled when loaded.
        /// </summary>
        ///
        /// <param name="nameSpace">not null and empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace or prefix is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if property values does not contain any values
        /// or any value is empty after trimmed or contains invalid regular expression. </exception>
        public RegexMatcher(string nameSpace, string prefix)
        {
            // verify the arguments
            FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");
            FileStatisticsHelper.CheckNull(prefix, "prefix");

            // get the property name
            string propertyName = (prefix == null) ?
                PatternsPropertyKey : prefix + "." + PatternsPropertyKey;

            try
            {
                ConfigManager cm = ConfigManager.GetInstance();

                // get the extensions from the config file
                string[] propertyValues = cm.GetValues(nameSpace, propertyName);
                if (propertyValues.Length == 0)
                {
                    // the property contains no values
                    throw new ConfigurationException(
                        "The property doesn't hold any values.");
                }

                // the variable to hold the patterns string
                // to check if overlapped pattern existed.
                IList tmpPatterns = new ArrayList();

                // add the extension to extensions
                foreach (string pattern in propertyValues)
                {
                    // if pattern is empty string, throw ConfigurationException
                    if (pattern.Trim().Length == 0)
                    {
                        throw new ConfigurationException(
                            "There are empty string existed in value of " + propertyName);
                    }

                    // if the patterns not existed, add it
                    if (!(tmpPatterns.Contains(pattern)))
                    {
                        tmpPatterns.Add(pattern);
                        this.patterns.Add(new Regex(pattern));
                    }
                }
            }
            catch(ConfigurationException)
            {
                throw;
            }
            catch(Exception e)
            {
                throw new ConfigurationException(
                    "Failed to get the correct patterns from the config file.", e);
            }
        }

        /// <summary>
        /// Check whether any of the patterns match the filename.
        /// </summary>
        ///
        /// <param name="filename">the filename to match</param>
        /// <returns>whether the filename matches</returns>
        protected override bool Match(string filename)
        {
            foreach (Regex pattern in patterns)
            {
                if (pattern.IsMatch(filename))
                {
                    // match found, return true
                    return true;
                }
            }

            // no match found, return false
            return false;
        }
    }
}