using System;
using System.IO;
using System.Collections;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Processor
{
    /// <summary>
    /// A base implementation that can be used for <see cref="IFileProcessor"/>.
    /// It provides alias and matcher support, including loading them from configuration file.
    /// It serves TextLineCounter and CStyleLineCounter as well as can be genericly used
    /// by future implementations.
    ///
    /// <configuration>
    /// This class can load configuration from a specified namespace
    /// The following properties are supported:
    /// - "alias" string value of the alias for this processor.
    /// This property is required and can not be empty.
    /// - "matcher" which specifies a file matcher. It contains a "classname" which
    /// specifies the fully qualified class name of a <see cref="IFileMatcher"/>
    /// implementation that owns a constructor with a property argument.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is immutable and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public abstract class FileProcessorBase : IFileProcessor
    {
        /// <summary>
        /// This property holds the alias for the file processor. It is required and can not be empty.
        /// </summary>
        public static readonly string AliasPropertyKey = "alias";

        /// <summary>
        /// This property contains sub-properties to specify the attached <see cref="IFileMatcher"/>
        /// implementation. When the implementation is instantiated the property itself is passed in
        /// as constructor argument. If not specified AnyFileMatcher is used.
        /// </summary>
        public static readonly string FileMatcherPropertyKey = "matcher";

        /// <summary>
        /// This is a sub-property in "matcher" to specify the fully qualified class name of the
        /// <see cref="IFileMatcher"/> implementation that owns a constructor with a Property argument.
        /// </summary>
        public static readonly string ClassnamePropertyKey = "classname";

        /// <summary>
        /// This is the alias for the filr processor. It is set in constructor and never changed.
        /// </summary>
        private string alias;

        /// <summary>
        /// This is the file matcher for the filr processor (and determines if a file can/should be
        /// processed). It is set in constructor and never changed.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// Get the alias for the file processor.
        /// </summary>
        ///
        /// <value>Represents the alias for the file processor.</value>
        public string Alias
        {
            get
            {
                return this.alias;
            }
        }

        /// <summary>
        /// Get the file matcher for the processor.
        /// </summary>
        ///
        /// <value>Represents the file matcher for the processor.</value>
        public IFileMatcher FileMatcher
        {
            get
            {
                return this.matcher;
            }
        }

        /// <summary>
        /// Get a list of all the supported metrics by this processor.
        /// This method is mostly for end user.
        /// It does not fulfill any responsibility in the current framework.
        /// </summary>
        ///
        /// <value>Represents the list of all the supported metrics.</value>
        public abstract IList SupportedMetrics
        {
            get;
        }

        /// <summary>
        /// Constructor with both alias and matcher. Set them to variables respectively.
        /// </summary>
        ///
        /// <param name="alias">the alias for the processor</param>
        /// <param name="matcher">the file matcher</param>
        ///
        /// <exception cref="ArgumentNullException">if alias or matcher is null</exception>
        /// <exception cref="ArgumentException">if alias is empty after trimmed</exception>
        protected FileProcessorBase(string alias, IFileMatcher matcher)
        {
            FileStatisticsHelper.CheckNullOrEmptyString(alias, "alias");
            FileStatisticsHelper.CheckNull(matcher, "matcher");

            this.alias = alias;
            this.matcher = matcher;
        }

        /// <summary>
        /// This constructor loads configuration from the property. The non-empty alias and
        /// non-null file matcher are required. Any error will be wrapped in a
        /// <see cref="ConfigurationException"/>.
        /// </summary>
        ///
        /// <param name="nameSpace">not null and empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace or prefix is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if config error occurs</exception>
        protected FileProcessorBase(string nameSpace, string prefix)
        {
            // verify the arguments
            FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");
            FileStatisticsHelper.CheckNull(prefix, "prefix");

            try
            {
                ConfigManager cm = ConfigManager.GetInstance();
                string propertyPrefix = (prefix == null) ?
                    "" : prefix + FileStatisticsHelper.CHAR_DOT;

                // get the alias property and verify the property value
                this.alias = cm.GetValue(nameSpace, propertyPrefix + AliasPropertyKey);
                FileStatisticsHelper.CheckPropertyValue(this.alias, "alias");

                // get the matcher class and verify the property value
                string matcherClass = cm.GetValue(nameSpace, propertyPrefix + FileMatcherPropertyKey
                    + FileStatisticsHelper.CHAR_DOT + ClassnamePropertyKey);
                FileStatisticsHelper.CheckPropertyValue(matcherClass, "matcherClass");

                // create instance of matcher
                string[] args = new string[] {nameSpace, propertyPrefix + FileMatcherPropertyKey};
                this.matcher = (IFileMatcher) FileStatisticsHelper.CreateInsance(matcherClass, args, null);
            }
            catch(ConfigurationException)
            {
                throw;
            }
            catch(Exception e)
            {
                throw new ConfigurationException("Failed to load the config value.", e);
            }
        }

        /// <summary>
        /// Process the file and generate the result. A new result should be generated each time,
        /// whose keys are the metrics returned from <see cref="SupportedMetrics"/>.
        /// </summary>
        ///
        /// <param name="file">the file to process</param>
        /// <returns>the stats result</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        /// <exception cref="FileProcessingException">if processor fails</exception>
        public abstract StatsResult Process(FileInfo file);
    }
}