using System;
using System.IO;
using System.Collections;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// <see cref="CStyleLineCounter"/> counts lines for C style source files.
    /// This implementation can handle C, C++, C# or Java. It only takes braces and
    /// comma into account so it would not be very accurate in most cases.
    /// Comments and string literals can be handled. It supports a single metric of "Line Count".
    ///
    /// <configuration>
    /// It delegates configuration to base class without any addition.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is stateless while its base class is thread-safe (thus it is thread-safe too).
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class CStyleLineCounter : FileProcessorBase
    {
        /// <summary>
        /// The name of the metric used for this processor.
        /// </summary>
        public static readonly string Name = "Line Count";

        /// <summary>
        /// The display name of the metric used for this processor.
        /// </summary>
        public static readonly string DisplayName = "line";

        /// <summary>
        /// The description of the metric used for this processor.
        /// </summary>
        private static readonly string Description = "CStyle Line counter";

        /// <summary>
        /// The supported metric list, contain a single metric.
        /// </summary>
        private IList supportedMetrices = new ArrayList();

        /// <summary>
        /// Return a list contains single Metric composed of the <see cref="Name"/>,
        /// <see cref="DisplayName"/> and a description.
        /// </summary>
        ///
        /// <value>Represents the list of supported metric.</value>
        public override IList SupportedMetrics
        {
            get
            {
                return this.supportedMetrices;
            }
        }

        /// <summary>
        /// Propagate to super constructor.
        /// </summary>
        ///
        /// <param name="alias">the alias</param>
        /// <param name="matcher">the file matcher</param>
        ///
        /// <exception cref="ArgumentNullException">if alias or matcher is null</exception>
        /// <exception cref="ArgumentException">if alias is empty after trimmed</exception>
        public CStyleLineCounter(string alias, IFileMatcher matcher) : base(alias, matcher)
        {
            Initialize();
        }

        /// <summary>
        /// Propagate to super constructor.
        /// </summary>
        ///
        /// <param name="nameSpace">non-empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if config error occurs</exception>
        public CStyleLineCounter(string nameSpace, string prefix) : base(nameSpace, prefix)
        {
            Initialize();
        }

        /// <summary>
        /// Count the lines in c style source code file.
        /// </summary>
        ///
        /// <param name="file">the file to process</param>
        /// <returns>the line count result</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        public override StatsResult Process(FileInfo file)
        {
            FileStatisticsHelper.CheckNull(file, "file");

            StreamReader reader = null;
            SimpleStateMachine state = null;
            try
            {
                // Create a text stream.
                reader = file.OpenText();

                // Create a SimpleStateMachine.
                state = new SimpleStateMachine();

                // Process each line with the state machine.
                string line = reader.ReadToEnd();
                state.Process(line);

                // Create a StatsResult and place the line count under the specified metric
                StatsResult result = new StatsResult();
                result[(Metric) this.supportedMetrices[0]] = state.Lines;

                return result;
            }
            catch(Exception e)
            {
                throw new IOException("Failed to process the file.", e);
            }
            finally
            {
                if (reader != null)
                {
                    // Close the stream
                    reader.Close();
                }
            }
        }

        /// <summary>
        /// Initialize the supported metric.
        /// </summary>
        private void Initialize()
        {
            Metric metric = new Metric(Name, DisplayName, Description);
            this.supportedMetrices.Add(metric);
        }
    }
}
