using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Processor.LineCounter;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Unit Test for <see cref="FileStatistics"/> class.
    /// It mainly tests the following:
    /// constructor: FileStatistics(), FileStatistics(string)
    /// static member: ProcessorsPropertyKey, ReportingPropertyKey, MatchAllPropertyKey, ClassnamePropertyKey
    /// property: AllFileProcessors, Reporting, IsMatchAll
    /// member: indexer, AddFileProcessor(IFileProcessor), RemoveFileProcessor(string),
    /// ClearAllFileProcessors(), ProcessFile(FileInfo)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class FileStatisticsUnitTest
    {
        /// <summary>
        /// Represents an instance of <see cref="FileStatistics"/>, used to test it's public member.
        /// </summary>
        private FileStatistics statistics;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);

            statistics = new FileStatistics();
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
        }

        /// <summary>
        /// Test static member ProcessorsPropertyKey, it should be "processors".
        /// </summary>
        [Test]
        public void TestProcessorsPropertyKey()
        {
            Assert.AreEqual("processors", FileStatistics.ProcessorsPropertyKey,
                "ProcessorsPropertyKey of FileStatistics shoud be 'processors'.");
        }

        /// <summary>
        /// Test static member ReportingPropertyKey, it should be "reporting".
        /// </summary>
        [Test]
        public void TestReportingPropertyKey()
        {
            Assert.AreEqual("reporting", FileStatistics.ReportingPropertyKey,
                "ReportingPropertyKey of FileStatistics shoud be 'reporting'.");
        }

        /// <summary>
        /// Test static member MatchAllPropertyKey, it should be "matchall".
        /// </summary>
        [Test]
        public void TestMatchAllPropertyKey()
        {
            Assert.AreEqual("matchall", FileStatistics.MatchAllPropertyKey,
                "MatchAllPropertyKey of FileStatistics shoud be 'matchall'.");
        }

        /// <summary>
        /// Test static member ClassnamePropertyKey, it should be "classname".
        /// </summary>
        [Test]
        public void TestClassnamePropertyKey()
        {
            Assert.AreEqual("classname", FileStatistics.ClassnamePropertyKey,
                "ClassnamePropertyKey of FileStatistics shoud be 'classname'.");
        }

        /// <summary>
        /// Test ctor FileStatistics(), instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(new FileStatistics(), "Failed to create instance of FileStatistics.");
        }

        /// <summary>
        /// Test ctor FileStatistics(string nameSpace), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_NameSpaceIsNull()
        {
            new FileStatistics(null);
        }

        /// <summary>
        /// Test ctor FileStatistics(string nameSpace), with nameSpace is null,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_NameSpaceIsEmpty()
        {
            new FileStatistics("  ");
        }

        /// <summary>
        /// Test ctor FileStatistics(string nameSpace), nameSpace is loaded and correct,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            Assert.IsNotNull(new FileStatistics(UnitTestHelper.DEFAULT_NAMESPACE),
                "Failed to create instance of FileStatistics.");
        }

        /// <summary>
        /// Test property AllFileProcessors Getter, no processor is added,
        /// emtpy list is expected.
        /// </summary>
        [Test]
        public void TestAllFileProcessors_Getter_Empty()
        {
            Assert.AreEqual(0, statistics.AllFileProcessors.Count,
                "No processor is added, empty list is expected.");
        }

        /// <summary>
        /// Test property AllFileProcessors Getter, add two processor
        /// and the added processor list is expected
        /// </summary>
        [Test]
        public void TestAllFileProcessors_Getter()
        {
            IFileProcessor processor1 = new TextLineCounter("Text", new AnyFileMatcher());
            IFileProcessor processor2 = new TextLineCounter("Sshape", new AnyFileMatcher());
            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);

            IList processors = statistics.AllFileProcessors;
            Assert.AreEqual(2, processors.Count, "The added two processor existed");
            Assert.IsTrue(processors.Contains(processor1),
                "Failed to get the correct processors.");
            Assert.IsTrue(processors.Contains(processor2),
                "Failed to get the correct processors.");
        }

        /// <summary>
        /// Test property Reporting Getter, default is <see cref="XmlStatsReporting"/>.
        /// </summary>
        [Test]
        public void TestReporting_Getter()
        {
            Assert.IsTrue(statistics.Reporting is XmlStatsReporting,
                "Default report type is XmlStatsReporting.");
        }

        /// <summary>
        /// Test property Reporting Setter, change the report to Null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestReporting_Setter_Null()
        {
            statistics.Reporting = null;
        }

        /// <summary>
        /// Test property Reporting Setter, change the report and verify it.
        /// </summary>
        [Test]
        public void TestReporting_Setter()
        {
            IStatsReporting report = new BasicStatsReporting();
            statistics.Reporting = report;

            Assert.AreEqual(report, statistics.Reporting,
                "Failed to set the Reporting to new value.");
        }

        /// <summary>
        /// Test property IsMatchAll Getter, default is false.
        /// </summary>
        [Test]
        public void TestIsMatchAll_Getter()
        {
            Assert.IsFalse(statistics.IsMatchAll, "Default match all should be false.");
        }

        /// <summary>
        /// Test property IsMatchAll Setter, set to new false, and verify it.
        /// </summary>
        [Test]
        public void TestIsMatchAll_Setter()
        {
            statistics.IsMatchAll = true;
            Assert.IsTrue(statistics.IsMatchAll, "Failed to change IsMatchAll to new value.");
        }

        /// <summary>
        /// Test indexer this[string alias], with alias is Null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexer_AliasIsNull()
        {
            IFileProcessor processor = statistics[null];
        }

        /// <summary>
        /// Test indexer this[string alias], with alias is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestIndexer_AliasIsEmpty()
        {
            IFileProcessor processor = statistics["  "];
        }

        /// <summary>
        /// Test indexer this[string alias], with alias is empty string,
        /// AE is expected.
        /// </summary>
        [Test]
        public void TestIndexer()
        {
            Assert.IsNull(statistics["Text"], "No such processor is existed, null should be returned.");

            IFileProcessor processor = new TextLineCounter("Text", new AnyFileMatcher());
            statistics.AddFileProcessor(processor);
            Assert.AreEqual(processor, statistics["Text"],
                "Failed to get the correct processor.");
        }

        /// <summary>
        /// Test AddFileProcessor(IFileProcessor processor), with processor is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestAddFileProcessor_ProcessorIsNull()
        {
            statistics.AddFileProcessor(null);
        }

        /// <summary>
        /// Test AddFileProcessor(IFileProcessor processor), add the processor,
        /// and verify the added.
        /// </summary>
        [Test]
        public void TestAddFileProcessor_NotExisted()
        {
            IFileProcessor processor = new TextLineCounter("Text", new AnyFileMatcher());
            statistics.AddFileProcessor(processor);

            Assert.AreEqual(1, statistics.AllFileProcessors.Count,
                "Only one processore existed");
            Assert.IsTrue(statistics.AllFileProcessors.Contains(processor),
                "Failed to add file processor.");
        }

        /// <summary>
        /// Test AddFileProcessor(IFileProcessor processor), add the processor,
        /// and verify the added.
        /// </summary>
        [Test]
        public void TestAddFileProcessor_Existed()
        {
            IFileProcessor processor1 = new TextLineCounter("Text", new AnyFileMatcher());
            IFileProcessor processor2 = new TextLineCounter("Text", new ExtensionMatcher("txt"));
            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);

            IList processors = statistics.AllFileProcessors;

            // verfiy the add procesor
            Assert.AreEqual(1, processors.Count,
                "Only one processore existed");
            Assert.IsFalse(processors.Contains(processor1),
                "Failed to add file processor, when processor is existed.");
            Assert.IsTrue(processors.Contains(processor2),
                "Failed to add file processor, when processor is existed.");
        }

        /// <summary>
        /// Test RemoveFileProcessor(string alias), with alias is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveFileProcessor_AliasIsNull()
        {
            statistics.RemoveFileProcessor(null);
        }

        /// <summary>
        /// Test RemoveFileProcessor(string alias), with alias is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestRemoveFileProcessor_AliasIsEmpty()
        {
            statistics.RemoveFileProcessor("  ");
        }

        /// <summary>
        /// Test RemoveFileProcessor(string alias), with alias is not existed,
        /// null should be returned.
        /// </summary>
        [Test]
        public void TestRemoveFileProcessor_AliasNotExisted()
        {
            Assert.IsNull(statistics.RemoveFileProcessor("NotExisted"),
                "Alias is not existed, null should be returned.");
        }

        /// <summary>
        /// Test RemoveFileProcessor(string alias), with alias is existed,
        /// the processor should be returned and removed.
        /// </summary>
        [Test]
        public void TestRemoveFileProcessor()
        {
            // add one processor
            IFileProcessor processor1 = new TextLineCounter("Text", new AnyFileMatcher());
            statistics.AddFileProcessor(processor1);

            // verify the added processor
            Assert.AreEqual(1, statistics.AllFileProcessors.Count,"Failed to add processor.");

            // remove the processor
            IFileProcessor returned = statistics.RemoveFileProcessor("Text");

            // verify the remove
            Assert.AreEqual(processor1, returned, "The returned processor is not expected.");
            Assert.AreEqual(0, statistics.AllFileProcessors.Count,"Failed to add processor.");
        }

        /// <summary>
        /// Test ClearAllFileProcessors(), all processors should be removed.
        /// </summary>
        [Test]
        public void TestClearAllFileProcessors()
        {
            // add file processor
            IFileProcessor processor1 = new TextLineCounter("C", new ExtensionMatcher("c"));
            IFileProcessor processor2 = new TextLineCounter("CShape", new ExtensionMatcher("cs"));
            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);

            // verfiy the add procesor
            Assert.AreEqual(2, statistics.AllFileProcessors.Count, "Failed to add file processor");

            statistics.ClearAllFileProcessors();

            // Failed to clear file processor
            Assert.AreEqual(0, statistics.AllFileProcessors.Count, "Failed to clear file processor");
        }

        /// <summary>
        /// Test ProcessFile(FileInfo file), with file is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestProcessFile_FileIsNull()
        {
            statistics.ProcessFile(null);
        }

        /// <summary>
        /// Test ProcessFile(FileInfo file), with file is not existed,
        /// IOException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(IOException))]
        public void TestProcessFile_FileNotExisted()
        {
            statistics.ProcessFile(new FileInfo("../../test_files/NotExisted"));
        }

        /// <summary>
        /// Test ProcessFile(FileInfo file), with processor is not existed.
        /// </summary>
        [Test]
        public void TestProcessFile_NoProcessor()
        {
            // set reprot to basic stats report
            statistics.Reporting = new BasicStatsReporting();

            // process file not supported
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text1.c"));

            // verify the generated report
            UnitTestHelper.VerifyResult(statistics.Reporting.GenerateReport(),
                "../../test_files/Reporting/statistics1.txt",
                "The report is not expected.");
        }

        /// <summary>
        /// Test ProcessFile(FileInfo file), with processor is not existed.
        /// </summary>
        [Test]
        public void TestProcessFile1()
        {
            statistics.Reporting.Reset(new DirectoryInfo("../../test_files"));
            // create some processors
            IFileProcessor processor1 = new TextLineCounter("Text", new ExtensionMatcher("txt"));
            IFileProcessor processor2 = new TextLineCounter("CShape", new ExtensionMatcher("cs"));
            IFileProcessor processor3 = new TextLineCounter("C", new ExtensionMatcher("c"));

            // add some processor
            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);
            statistics.AddFileProcessor(processor3);

            // process some files
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text1.c"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text2.c"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text3.c"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text4.c"));

            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text1.txt"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text2.txt"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text3.txt"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text4.txt"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text5.txt"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/sub/text4.txt"));
            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/sub/text5.txt"));

            statistics.ProcessFile(new FileInfo("../../test_files/Processor/LineCounter/text12.cs"));

            // verify the generated report
            UnitTestHelper.VerifyResult(statistics.Reporting.GenerateReport(),
                "../../test_files/Reporting/statistics2.txt",
                "The report is not expected.");
        }
    }
}