using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Unit test for <see cref="ExtensionMatcher"/> class.
    /// It mainly tests the following:
    /// static member: ExtensionsPropertyKey
    /// constructor: ExtensionMatcher(string), ExtensionMatcher(IList), ExtensionMatcher(string, string)
    /// member: Match(string)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class ExtensionMatcherUnitTest
    {
        /// <summary>
        /// Set up for each test.
        /// Load config file.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.INVALID_CONFIG);
        }

        /// <summary>
        /// Tear down for each test.
        /// Remove config files.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
        }

        /// <summary>
        /// Test public static member ExtensionsPropertyKey, it should be 'extensions'.
        /// </summary>
        [Test]
        public void TestExtensionsPropertyKey()
        {
            Assert.AreEqual("extensions", ExtensionMatcher.ExtensionsPropertyKey,
                "ExtensionsPropertyKey should be 'extensions'");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string extension), with extension is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_ExtensionIsNull()
        {
            new ExtensionMatcher((string) null);
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string extension), with extension is empty string,
        /// allowed
        /// </summary>
        [Test]
        public void TestCtor1_ExtensionIsEmpty()
        {
            ExtensionMatcher matcher = new ExtensionMatcher("  ");
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/noExtension")),
                "With extension is empty, file with extension should be matched.");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(IList extensions), with extensions is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_ExtensionsIsNull()
        {
            new ExtensionMatcher((IList) null);
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(IList extensions), with extensions contains null,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_ExtensionsContainsNull()
        {
            IList extensions = new ArrayList();
            extensions.Add(null);
            extensions.Add("cs");

            new ExtensionMatcher((IList) extensions);
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(IList extensions), with extensions contains non-string instance,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_ExtensionsContainsNonString()
        {
            IList extensions = new ArrayList();
            extensions.Add(new object());
            extensions.Add("cs");

            new ExtensionMatcher((IList) extensions);
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(IList extensions), with extensions contains emtpy string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_ExtensionsContainsEmptyString()
        {
            IList extensions = new ArrayList();
            extensions.Add("  ");
            extensions.Add("cs");

            new ExtensionMatcher((IList) extensions);
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(IList extensions), with extensions contains extension,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            IList extensions = new ArrayList();
            extensions.Add("java");
            extensions.Add("cs");

            IFileMatcher matcher = new ExtensionMatcher((IList) extensions);
            Assert.IsNotNull(matcher, "Failed to create instance of ExtensionMatcher");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string nameSpace, string prefix), with namespace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_NamespaceIsNull()
        {
            new ExtensionMatcher(null, "processor1.matcher");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string nameSpace, string prefix), with namespace is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor3_NamespaceIsEmpty()
        {
            new ExtensionMatcher("  ", "processor1.matcher");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string nameSpace, string prefix), with prefix is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_PrefixIsNull()
        {
            new ExtensionMatcher(UnitTestHelper.DEFAULT_NAMESPACE, null);
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError1()
        {
            new ExtensionMatcher(UnitTestHelper.INVALID_NAMESPACE, "ExtensionMatcher1");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError2()
        {
            new ExtensionMatcher(UnitTestHelper.INVALID_NAMESPACE, "ExtensionMatcher2");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError3()
        {
            new ExtensionMatcher(UnitTestHelper.INVALID_NAMESPACE, "ExtensionMatcher3");
        }

        /// <summary>
        /// Test ctor ExtensionMatcher(string nameSpace, string prefix),
        /// with both argument are valid, instance should be created.
        /// </summary>
        public void TestCtor3()
        {
            IFileMatcher matcher = new ExtensionMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor1.matcher");
            Assert.IsNotNull(matcher, "Failed to create instance of ExtensionMatcher");
        }

        /// <summary>
        /// Test Match(string filename), since it is protected, test Match(FileInfo file) instead.
        /// the default config file suported two extension "txt" and "sql".
        /// </summary>
        public void TestMatch_True()
        {
            IFileMatcher matcher = new ExtensionMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor1.matcher");
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/test.sql")),
                "sql file should be matched.");
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/test.Txt")),
                "txt file should be matched.");
        }

        /// <summary>
        /// Test Match(string filename), since it is protected, test Match(FileInfo file) instead.
        /// the default config file suported two extension "txt" and "sql".
        /// </summary>
        public void TestMatch_False()
        {
            IFileMatcher matcher = new ExtensionMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor1.matcher");
            Assert.IsFalse(matcher.Match(new FileInfo("../../test_files/matcher/test.pl")),
                "pl file should not be matched.");
        }
    }
}