using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Unit test for abstract <see cref="FileCategoryMatcher"/> class,
    /// use <see cref="MockFileCategoryMatcher"/> instead.
    /// It mainly tests the following:
    /// constructor: FileCategoryMatcher()
    /// static member: IsBinary(FileInfo)
    /// member: Match(FileInfo)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class FileCategoryMatcherUnitTest
    {
        /// <summary>
        /// Represents an instance of <see cref="FileCategoryMatcher"/>, used to test it's public member.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            matcher = new MockFileCategoryMatcher();
        }

        /// <summary>
        /// Test ctor FileCategoryMatcher(), instance should be created.
        /// </summary>
        [Test]
        public void TestCtor()
        {
            Assert.IsNotNull(matcher, "Failed to create instance of FileCategoryMatcher");
        }

        /// <summary>
        /// Test static member IsBinary(FileInfo file), with file is null,
        /// ANE is expected
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestIsBinary_FileIsNull()
        {
            FileCategoryMatcher.IsBinary(null);
        }

        /// <summary>
        /// Test static member IsBinary(FileInfo file), with file is not existed.
        /// IOException is expected
        /// </summary>
        [Test, ExpectedException(typeof(IOException))]
        public void TestIsBinary_FileNotExisted()
        {
            FileCategoryMatcher.IsBinary(new FileInfo("../../test_files/matcher/NotExisted"));
        }

        /// <summary>
        /// Test static member IsBinary(FileInfo file), with file is text file
        /// false is expected
        /// </summary>
        [Test]
        public void TestIsBinary_TextFile1()
        {
            Assert.IsFalse(FileCategoryMatcher.IsBinary(new FileInfo("../../test_files/matcher/test.txt")),
                "Failed to determine the correct file category");
        }

        /// <summary>
        /// Test static member IsBinary(FileInfo file), with file is text file
        /// false is expected
        /// </summary>
        [Test]
        public void TestIsBinary_TextFile2()
        {
            Assert.IsFalse(FileCategoryMatcher.IsBinary(new FileInfo("../../test_files/matcher/test.big.txt")),
                "Failed to determine the correct file category");
        }


        /// <summary>
        /// Test static member IsBinary(FileInfo file), with file is binary file
        /// true is expected
        /// </summary>
        [Test]
        public void TestIsBinary_BinFile1()
        {
            Assert.IsTrue(FileCategoryMatcher.IsBinary(new FileInfo("../../test_files/matcher/test.bin")),
                "Failed to determine the correct file category");
        }

        /// <summary>
        /// Test static member IsBinary(FileInfo file), with file is binary file
        /// true is expected
        /// </summary>
        [Test]
        public void TestIsBinary_BinFile2()
        {
            Assert.IsTrue(FileCategoryMatcher.IsBinary(new FileInfo("../../test_files/matcher/test.big.bin")),
                "Failed to determine the correct file category");
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is null, ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestMatch_FileIsNull()
        {
            matcher.Match(null);
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is not existed.
        /// IOException is expected
        /// </summary>
        [Test, ExpectedException(typeof(IOException))]
        public void TestMatch_FileNotExisted()
        {
            matcher.Match(new FileInfo("../../test_files/matcher/NotExisted"));
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is text file,
        /// false is expected.
        /// </summary>
        [Test]
        public void TestMatch_TextFile1()
        {
            Assert.IsFalse(matcher.Match(new FileInfo("../../test_files/matcher/test.txt")),
                "Failed to determine the correct file category");
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is text file,
        /// false is expected
        /// </summary>
        [Test]
        public void TestMatch_TextFile2()
        {
            Assert.IsFalse(matcher.Match(new FileInfo("../../test_files/matcher/test.big.txt")),
                "Failed to determine the correct file category");
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is binary file
        /// true is expected
        /// </summary>
        [Test]
        public void TestMatch_BinFile1()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/test.bin")),
                "Failed to determine the correct file category");
        }

        /// <summary>
        /// Test Match(FileInfo file), with file is binary file
        /// true is expected
        /// </summary>
        [Test]
        public void TestMatch_BinFile2()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/test.big.bin")),
                "Failed to determine the correct file category");
        }
    }
}