using System.IO;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using TopCoder.File.Statistics.Reporting;

namespace TopCoder.File.Statistics
{	
	/// <summary>
	/// This is the main class of the user API. It provides the basic API, which includes
	/// maintaining a list of file processors, a reporting handler and controls the way
	/// to process the file. Files are processed one by one and the results are aggregated
	/// into the reporting. If you want to start a new reporting you should either
	/// explicitly set the reporting or get the old reporting to reset it.
	/// 
	/// There are two ways to process the file:
	/// - Process it with the first matching processor.
	/// - Process it with each martching processor.
	/// Refer to <see cref="ExtendedFileStatistics"/> for advanced usage.
	/// 
	/// <configuration>
	/// This class can be intialized from a configuration namespace which should be preloaded.
	/// It will look at the following properties:
	/// - "processors" as a container for file processors, each sub-property should contain
	/// a "classname" that specifies the fully qualified class name of an
	/// <see cref="IFileProcessor"/> implementation which accepts a <see cref="Property"/> as
	/// constructor argument. This property is required.
	/// 
	/// - "reporting" which contains a "classname" that specifies the fully qualified class
	/// name of an <see cref="IStatsReporting"/> implementation which accepts a
	/// <see cref="Property"/> as constructor argument. This property is optional
	/// (<see cref="XmlStatsReporting"/> is the default value).
	/// 
	/// - "matchall" which is a boolean flag that indicates whether all matching processors
	/// will be used to process the file. It is regarded true when value is either "on",
	/// "yes", "true" case-insensitively. This property is optional.
	/// </configuration>
	/// 
	/// <thread-safety>
	/// This class is not thread-safe in that it provides a number of manipulation methods
	/// in a non-synchronized manner. Even if calling from different threads at separated
	/// time will mix the result into the same reporting, which may not be the desirable
	/// behavior.
	/// </thread-safety>
	/// 
	/// <author>WishingBone</author>
	/// <author>TCSDEVELOPER</author>
	/// <version>1.0</version>
	/// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
	/// </summary>
	public class FileStatistics 
	{	
		/// <summary>
		/// This property contains multiple sub-properties to specify
		/// <see cref="IFileProcessor"/> implementations. When the implementation is
		/// instantiated the property itself is passed in as constructor argument.
		/// </summary>
		public static readonly string ProcessorsPropertyKey = "processors";

		/// <summary>
		/// This property contains sub-properties to specify the <see cref="IStatsReporting"/>
		/// implementation. When the implementation is instantiated the property itself
		/// is passed in as constructor argument.
		/// </summary>
		public static readonly string ReportingPropertyKey = "reporting";

		/// <summary>
		/// This property specifies whether a file is processed by the first matching processor
		/// or each of the matching processor. This property is optional (default to false).
		/// Its value is considered true if it's "true", "yes" or "on".
		/// </summary>
		public static readonly string MatchAllPropertyKey = "matchall";

		/// <summary>
		/// This is a sub-property in "processors"' sub-properties or "reporting" to specify
		/// the fully qualified class name of the <see cref="IFileProcessor"/> or
		/// <see cref="IStatsReporting"/> implementation that owns a constructor with
		/// a <see cref="Property"/> argument.
		/// </summary>
		public static readonly string ClassnamePropertyKey = "classname";

		/// <summary>
		/// Represents the property name of "names".
		/// </summary>
		private const string PROPERTY_NAMES = "names";

		/// <summary>
		/// Represents the property name of "stylesheet".
		/// </summary>
		private const string PROPERTY_STYLESHEET = "stylesheet";

		/// <summary>
		/// Represents the property name of "alias".
		/// </summary>
		private const string PROPERTY_ALIAS = "alias";

		/// <summary>
		/// Represents the property name of "matcher".
		/// </summary>
		private const string PROPERTY_MATCHER = "matcher";

		/// <summary>
		/// Represents the property name of "extensions".
		/// </summary>
		private const string PROPERTY_EXTENSIONS = "extensions";

		/// <summary>
		/// Represents the property name of "types".
		/// </summary>
		private const string PROPERTY_TYPES = "types";

		/// <summary>
		/// The processors aggregated. 
		/// This is intialized to a hashtable and the reference will never change.
		/// The key is the processor name and the value is an instance of
		/// <see cref="IFileProcessor"/> processor.
		/// </summary>
		private IDictionary processors = new Hashtable();

		/// <summary>
		/// The reporting to use. This is mutable.
		/// </summary>
		private IStatsReporting reporting = new XmlStatsReporting();

		/// <summary>
		/// A flag on whether to process the file with each matching processor.
		/// </summary>
		private bool matchAll = false;

		/// <summary>
		/// Get a list of all the processors aggregated. 
		/// Should return a copy of the inner collection.
		/// </summary>
		/// 
		/// <value>Represents the processor list.</value>
		public IList AllFileProcessors
		{
			get
			{
				return new ArrayList(this.processors.Values);
			}
		}

		/// <summary>
		/// Get or set the reporting instance. Simply return the variable.
		/// </summary>
		/// 
		/// <value>Represents the reporting instance.</value>
		/// 
		/// <exception cref="ArgumentNullException">if reporting is null</exception>
		public IStatsReporting Reporting
		{
			get
			{
				return this.reporting;
			}
			set
			{
				FileStatisticsHelper.CheckNull(value, "Reporting");
				this.reporting = value;
			}
		}

		/// <summary>
		/// Get or set whether to process the file with each matching processor.
		/// </summary>
		/// 
		/// <value>Represents the flag on whether to process the file with each matching
		/// processor.</value>
		public bool IsMatchAll
		{
			get
			{
				return this.matchAll;
			}
			set
			{
				this.matchAll = value;
			}
		}

		/// <summary>
		/// Get the file processor with given alias. If no such processor exists return null.
		/// </summary>
		/// 
		/// <param name="alias">the alias of the processor</param>
		/// <returns>the associated file processor</returns>
		/// 
		/// <exception cref="ArgumentNullException">if alias is null</exception>
		/// <exception cref="ArgumentException">if alias is empty after trimmed</exception>
		public IFileProcessor this[string alias]
		{
			get
			{
				FileStatisticsHelper.CheckNullOrEmptyString(alias, "alias");
				return (IFileProcessor) this.processors[alias];
			}
		}

		/// <summary>
		/// Default constructor. Does nothing.
		/// </summary>
		public FileStatistics()
		{
		}
		
		/// <summary>
		/// Constructor to load configuration from given namespace.
		/// The namespace should be already loaded.
		/// It loads the processors, the reporting as well as the matchall flag.
		/// </summary>
		/// 
		/// <param name="nameSpace">the namespace to load from</param>
		/// 
		/// <exception cref="ArgumentNullException">if namespace is null</exception>
		/// <exception cref="ArgumentException">if namespace is empty after trimmed</exception>
		/// <exception cref="ConfigurationException">if config error occurs</exception>
		public FileStatistics(string nameSpace)
		{
			FileStatisticsHelper.CheckNullOrEmptyString(nameSpace, "nameSpace");

			ConfigManager cm = ConfigManager.GetInstance();

			// get processor
			string[] processorNames = cm.GetValues(nameSpace, 
				ProcessorsPropertyKey + FileStatisticsHelper.CHAR_DOT + PROPERTY_NAMES);
			foreach (string processorName in processorNames)
			{
				string processorClass = cm.GetValue(nameSpace, 
					processorName + FileStatisticsHelper.CHAR_DOT + ClassnamePropertyKey);
				string processorAlias = cm.GetValue(nameSpace,
					processorName + FileStatisticsHelper.CHAR_DOT + PROPERTY_ALIAS);
				string processorMatcher = cm.GetValue(nameSpace,
					processorName + FileStatisticsHelper.CHAR_DOT + PROPERTY_MATCHER + ClassnamePropertyKey);
				string[] processorMatcherExtentsions = cm.GetValues(nameSpace,
					processorName + FileStatisticsHelper.CHAR_DOT + PROPERTY_MATCHER + PROPERTY_EXTENSIONS);
				string[] processorMatcherTypes = cm.GetValues(
					nameSpace, processorName + FileStatisticsHelper.CHAR_DOT
					+ PROPERTY_MATCHER + FileStatisticsHelper.CHAR_DOT + PROPERTY_TYPES);
			}

			// get reporting
			string reportClass = cm.GetValue(nameSpace, 
				ReportingPropertyKey + FileStatisticsHelper.CHAR_DOT + ClassnamePropertyKey);
			string reportStyle = cm.GetValue(nameSpace, 
				ReportingPropertyKey + FileStatisticsHelper.CHAR_DOT + PROPERTY_STYLESHEET);

			// get matchAll
			string matchAllValue = cm.GetValue(nameSpace, MatchAllPropertyKey);
			matchAllValue = matchAllValue.ToLower();
			this.matchAll = matchAllValue.Equals("yes") 
				|| matchAllValue.Equals("true")  || matchAllValue.Equals("on");

		}

		/// <summary>
		/// Add the file processor to the aggregation. If there is a processor with
		/// the same alias just overwrite the old one.
		/// </summary>
		/// 
		/// <param name="processor">the file processor to add</param>
		/// 
		/// <exception cref="ArgumentNullException">if processor is null</exception>
		public void AddFileProcessor(IFileProcessor processor)
		{
			FileStatisticsHelper.CheckNull(processor, "processor");

			this.processors.Add(processor.Alias, processor);
		}
		
		/// <summary>
		/// Remove the file processor with given alias. If no such process exists return null.
		/// </summary>
		/// 
		/// <param name="alias">the alias of the processor</param>
		/// <returns>the processor just removed</returns>
		/// 
		/// <exception cref="ArgumentNullException">if alias is null</exception>
		/// <exception cref="ArgumentException">if alias is empty after trimmed</exception>
		public IFileProcessor RemoveFileProcessor(string alias) 
		{
			IFileProcessor processor = (IFileProcessor) this[alias];
			this.processors.Remove(alias);

			return processor;
		}

		/// <summary>
		/// Clear all the aggregated processors.
		/// </summary>
		public void ClearAllFileProcessors() 
		{
			this.processors.Clear();
		}

		/// <summary>
		/// Process a single file.
		/// </summary>
		/// 
		/// <param name="file">the file to process</param>
		/// 
		/// <exception cref="ArgumentNullException">if file is null</exception>
		/// <exception cref="IOException">if io operation fails</exception>
		/// <exception cref="FileStatisticsException">if processing fails</exception>
		public void ProcessFile(FileInfo file)
		{
			FileStatisticsHelper.CheckNull(file, "file");

			foreach (IFileProcessor processor in this.processors.Values)
			{
				if (processor.FileMatcher.Match(file))
				{
					// Process the file with the matched processor(s);
					StatsResult result = processor.Process(file);

					// Aggregate the result into reporting;
					this.reporting.Aggregate(processor, file, result);
					if (!this.IsMatchAll)
					{
						// If matchall is off, break;
						break;
					}
				}
			}
		}
	}
}