using System;
using System.IO;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// <see cref="FileCategoryMatcher"/> is a template class that handles file category (text/binary)
    /// oriented matching logic. The purpose to still create subclasses for such minor functionality
    /// is about the convenience in configuration.
    ///
    /// <thread-safety>
    /// This class is stateless and thread-safe.
    /// </thread-safety>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public abstract class FileCategoryMatcher : IFileMatcher
    {
        /// <summary>
        /// Represent the max bytes need to check whether the file is binary.
        /// </summary>
        private const int MAX_CHECK_SIZE = 1024;

        /// <summary>
        /// Represents the range begin of ascii characters.
        /// </summary>
        private const byte ASCII_BEGIN_RANGE = 32;

        /// <summary>
        /// Represents the range end of ascii characters.
        /// </summary>
        private const byte ASCII_END_RANGE = 127;

        /// <summary>
        /// Represents the ascii code of tab
        /// </summary>
        private const byte ASCII_TAB = 9;

        /// <summary>
        /// Represents the ascii code of carriage return.
        /// </summary>
        private const byte ASCII_CR = 13;

        /// <summary>
        /// Represents the ascii code of line feed.
        /// </summary>
        private const byte ASCII_LF = 10;
        
        /// <summary>
        /// Represents the ratio for determine file type.
        /// </summary>        
        private const int RATIO = 10;

        /// <summary>
        /// Default constructor. Does nothing.
        /// </summary>
        protected FileCategoryMatcher()
        {
        }

        /// <summary>
        /// It checks if the file is binary. This method is exposed as statuc public
        /// because it could potentially be used by other custom matchers.
        /// </summary>
        ///
        /// <param name="file">the file to check</param>
        /// <returns>whether the file is binary</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        public static bool IsBinary(FileInfo file)
        {
            // verify the file
            FileStatisticsHelper.CheckNull(file, "file");

            BinaryReader reader = null;
            int pos = 0;
            int nonAscii = 0;
            try
            {
                // open a binary reader
                reader = new BinaryReader(file.OpenRead());

                // check the file at most 1024 bytes
                while (pos < MAX_CHECK_SIZE)
                {
                    byte data = reader.ReadByte();                    
                    if (!(((data >= ASCII_BEGIN_RANGE) && (data <= ASCII_END_RANGE))
                        || (data == ASCII_TAB)
                        || (data == ASCII_CR)
                        || (data == ASCII_LF)))
                    {
                        ++nonAscii;
                    }
                    ++pos;
                }
                return (nonAscii * RATIO >= pos);
            }
            catch(EndOfStreamException)
            {
                // reach end of file, binary file.
                return (nonAscii * RATIO >= pos);
            }
            catch(ObjectDisposedException ode)
            {
                throw new IOException("Failed to check for the stream being closed.", ode);
            }
            catch(UnauthorizedAccessException uae)
            {
                throw new IOException("Failed to check for unauthorized access.", uae);
            }
            catch(DirectoryNotFoundException dnfe)
            {
                throw new IOException("Failed to check for directory not found.", dnfe);
            }
            catch(FileNotFoundException fnfe)
            {
                throw new IOException("The '" + file.FullName + "' is not existed.", fnfe);
            }
            catch(IOException)
            {
                throw;
            }
            catch(Exception e)
            {
                throw new IOException("Failed to check the file category.", e);
            }
            finally
            {
                if (reader != null)
                {
                    reader.Close();
                }
            }
        }

        /// <summary>
        /// It uses isBinary() to check if the file is binary, and passes the flag to
        /// the template method to decide.
        /// </summary>
        ///
        /// <param name="file">the file to match</param>
        /// <returns>whether the file matches</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        public bool Match(FileInfo file)
        {
            return Match(IsBinary(file));
        }

        /// <summary>
        /// Decide whether the file category matches.
        /// </summary>
        ///
        /// <param name="isBinary">whether the file is binary</param>
        /// <returns>whether the category matches</returns>
        protected abstract bool Match(bool isBinary);
    }
}
