using System;
using System.IO;
using System.Collections;
using TopCoder.Util.ConfigurationManager;

namespace TopCoder.File.Statistics.Processor.LineCounter
{
    /// <summary>
    /// <see cref="TextLineCounter"/> counts lines for text files.
    /// It supports a single metric of "Line Count".
    ///
    /// <configuration>
    /// It delegates configuration to base class without any addition.
    /// </configuration>
    ///
    /// <thread-safety>
    /// This class is stateless while its base class is thread-safe (thus it is thread-safe too).
    /// </thread-safety>
    /// 
    /// <changes>
    /// Since version 2.0b this class adds an option to whether ignore a blank line (line composed
    /// of whitespace only). This option can come from either configuration file or command line.
    /// </changes>
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>2.0b</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    public class TextLineCounter : FileProcessorBase, IArgumentHandler
    {
        /// <summary>
        /// Configuration property key that holds a boolean flag whether to
        /// ignore blank line. This is optional and is evaluated to true
        /// if the value is "true"/"yes"/"on" case sensitively.
        /// Default value is false.
        /// <since>2.0b</since>
        /// </summary>
        public static readonly string IgnoreBlankLinePropertyKey = "ignore_blank_line";
    
        /// <summary>
        /// The name of the metric used for this processor.
        /// </summary>
        public static readonly string Name = "Line Count";

        /// <summary>
        /// The display name of the metric used for this processor.
        /// </summary>
        public static readonly string DisplayName = "line";

        /// <summary>
        /// The description of the metric used for this processor.
        /// </summary>
        private static readonly string Description = "Text Line counter";
        
        /// <summary>
        /// Flag whether to ignore blank line. It's set in constructor and never changes.
        /// <since>2.0b</since>
        /// </summary>
        private readonly bool ignoreBlankLine = false;

        /// <summary>
        /// The supported metric list, contain a single metric.
        /// </summary>
        private IList supportedMetrices = new ArrayList();

        /// <summary>
        /// Return a list contains single <see cref="Metric"/> composed of the <see cref="Name"/>,
        /// <see cref="DisplayName"/> and a description.
        /// </summary>
        ///
        /// <value>Represents the list of the supported metric.</value>
        public override IList SupportedMetrics
        {
            get
            {
                return this.supportedMetrices;
            }
        }

        /// <summary>
        /// Propagate to super constructor.
        /// </summary>
        ///
        /// <param name="alias">the alias</param>
        /// <param name="matcher">the file matcher</param>
        ///
        /// <exception cref="ArgumentNullException">if alias or matcher is null</exception>
        /// <exception cref="ArgumentException">if alias is empty after trimmed</exception>
        public TextLineCounter(string alias, IFileMatcher matcher) : base(alias, matcher)
        {
            Initialize();
            this.ignoreBlankLine = false;
        }
        
        /// <summary>
        /// Propagate the super constructor and set ignoreBlankLine flag.
        /// <since>2.0b</since>
        /// </summary>
        /// 
        /// <param name="alias">the alias</param>
        /// <param name="matcher">the file matcher</param>
        /// <param name="ignoreBlankLine"></param>
        /// <exception cref="ArgumentNullException">if alias or matcher is null</exception>
        /// <exception cref="ArgumentException">if alias is empty after trimmed</exception>
        public TextLineCounter(string alias, IFileMatcher matcher, bool ignoreBlankLine) : base(alias, matcher)
        {
            Initialize();
            this.ignoreBlankLine = ignoreBlankLine;            
        }
        
        /// <summary>
        /// Propagate to super constructor.
        /// <changes>
        /// Since version 2.0b this adds the logic to load the ignore blank line property.
        /// </changes>
        /// </summary>
        ///
        /// <param name="nameSpace">not null and empty, the namespace to load config from.</param>
        /// <param name="prefix">
        /// can be empty, the prefix indicates the global property level.</param>
        ///
        /// <exception cref="ArgumentNullException">if nameSpace or prefix is null</exception>
        /// <exception cref="ArgumentException">if nameSpace is emtpy</exception>
        /// <exception cref="ConfigurationException">if config error occurs</exception>
        public TextLineCounter(string nameSpace, string prefix) : base(nameSpace, prefix)
        {
            Initialize();
            ConfigManager cm = ConfigManager.GetInstance();
            string propertyPrefix = (prefix == null) ? 
                IgnoreBlankLinePropertyKey : prefix + "." + IgnoreBlankLinePropertyKey;
            System.Console.WriteLine(propertyPrefix);
            
            // get the ignore_blank_line property and verify the property value
            string propertyValue = cm.GetValue(nameSpace, propertyPrefix);
            if (propertyValue != null) {
                this.ignoreBlankLine = FileStatisticsHelper.CheckBoolValue(propertyValue,
                    "the property value of ignore_blank_line is invalid");
            }
            else
            {
                this.ignoreBlankLine = false;
            }
        }

        /// <summary>
        /// Count the lines in text file.
        /// </summary>
        ///
        /// <param name="file">the file to process</param>
        /// <returns>the line count result</returns>
        ///
        /// <exception cref="ArgumentNullException">if file is null</exception>
        /// <exception cref="IOException">if io operation fails</exception>
        public override StatsResult Process(FileInfo file)
        {
            FileStatisticsHelper.CheckNull(file, "file");

            int lineCount = 0;

            StreamReader reader = null;
            try
            {
                // Create a text stream.
                reader = file.OpenText();

                // Count the lines until EOF is met
                if (this.ignoreBlankLine) {
                    string line;
                    while ((line = reader.ReadLine()) != null)
                    {
                        if (!IsBlankLine(line)) {
                            ++lineCount;
                        }
                    }
                } else {
                    while (reader.ReadLine() != null)
                    {
                        ++lineCount;
                    }
                }

                // Create a StatsResult and place the line count under the specified metric.
                StatsResult result = new StatsResult();
                result[(Metric) this.supportedMetrices[0]] = lineCount;

                return result;
            }
            catch(Exception e)
            {
                throw new IOException("Failed to process the file.", e);
            }
            finally
            {
                if (reader != null)
                {
                    // Close the stream
                    reader.Close();
                }
            }
        }
        
        /// <summary>
        /// Propose a single ArgumentOption with "ignore_blank_line" as switch name
        /// and required as false.
        /// </summary>
        /// <returns></returns>
        public IList ProposeOptions() {
            IList options = new ArrayList();
            options.Add(new ArgumentOption(IgnoreBlankLinePropertyKey, 
                "ignore blank line", false));
            return options;
        }
        
        /// <summary>
        /// Receive command line argument from the command line interface.
        /// It does not care about the actual value of the switch
        /// (which is generally not provided). It only cares about if the switch is on or off.
        /// </summary>
        /// 
        /// <param name="arguments">the arguments from command line</param>
        /// <exception cref="ArgumentNullException">if arguments is null</exception> 
        /// <exception cref="ArgumentExcpetion">if "ignore_blank_line"
        /// does not correspond to a String</exception> 
        public void HandleArguments(IDictionary arguments) {
            FileStatisticsHelper.CheckNull(arguments, "arguments");
            FileStatisticsHelper.CheckArgument(
                arguments, IgnoreBlankLinePropertyKey);
        }    

        /// <summary>
        /// Initialize the supported metric.
        /// </summary>
        private void Initialize()
        {
            Metric metric = new Metric(Name, DisplayName, Description);
            this.supportedMetrices.Add(metric);
        }
        
        /// <summary>
        /// Determine the line is blank or not, return true if it is, false otherwise.
        /// </summary>
        /// <param name="line">the line to check</param>
        private bool IsBlankLine(string line) {
            int i = 0;
            while (i < line.Length) {
                if (!(line[i] == ' ' || line[i] == '\t')) {
                    return true;
                }
                ++i;
            }
            return false;
        }
    }
}
