using System;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor;
using TopCoder.File.Statistics.Reporting;
using TopCoder.File.Statistics.Processor.LineCounter;

namespace TopCoder.File.Statistics
{
    /// <summary>
    /// Demo usage of the file statistics component.
    ///
    /// <author>WishingBone</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class Demo
    {
        /// <summary>
        /// Tear down for each demo.
        /// </summary>
        [TearDown]
        public void RunAfterEachDemo()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.COMMANDLINE_CONFIG);
            UnitTestHelper.DeleteFile("-o:../../test_files/output.html");
        }

        /// <summary>
        /// Demo usage, show how to config and operate the File Statistics
        /// </summary>
        [Test]
        public void Config()
        {
            // create a file statistics instance
            FileStatistics statistics = CreateStatistics();

            // remove file processor
            IFileProcessor removed = statistics.RemoveFileProcessor("Source File");

            // query file processor
            IFileProcessor query = statistics["Text File"];

            // get all processors
            IList processors = statistics.AllFileProcessors;

            // clear all processors
            statistics.ClearAllFileProcessors();

            // manipulate reporting
            IStatsReporting original = statistics.Reporting;
            IStatsReporting reporting2 = new XmlStatsReporting(
                new FileInfo("../../test_files/conf/reporting.xsl"));
            statistics.Reporting = reporting2;

            // manipulate match all flag
            if (!statistics.IsMatchAll)
            {
                statistics.IsMatchAll = true;
            }
        }

        /// <summary>
        /// Demo usage, show how to processing file or directory.
        /// </summary>
        [Test]
        public void ProcessFile()
        {
            // generate a statistics
            ExtendedFileStatistics statistics = (ExtendedFileStatistics) CreateStatistics();

            // process single file
            statistics.ProcessFile(new FileInfo("../../test_files/Reporting/basic1.txt"));
            statistics.ProcessFile(new FileInfo("../../test_files/Reporting/xml1.xml"));

            // process directory
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files/Processor"));
            statistics.ProcessDirectory(new DirectoryInfo("../../test_files"), true);
        }

        /// <summary>
        /// Demo usage, show how to generate report.
        /// </summary>
        [Test]
        public void GenerateReport()
        {
            // generate a statistics
            ExtendedFileStatistics statistics = (ExtendedFileStatistics) CreateStatistics();

            // obtain report
            string report = statistics.Reporting.GenerateReport();

            // output report
            statistics.PrintReport();
            statistics.PrintReport(new FileInfo("../../test_files/output.html"));

            // reset reporting
            statistics.Reporting.Reset(new DirectoryInfo("../../test_files"));
        }

        /// <summary>
        /// Demo usage, show how to use the component as command line tools.
        /// </summary>
        [Test]
        public void Command()
        {
            // set the console output to new
            MemoryStream stream = new MemoryStream(2048);
            TextWriter newOut = new StreamWriter(stream);
            TextWriter oldOut = Console.Out;
            Console.SetOut(newOut);

            // specify configuration file
            string[] args = new string[] {"-c:" + UnitTestHelper.COMMANDLINE_CONFIG, "../../test_files/Processor"};
            CommandLine.Main(args);

            UnitTestHelper.RemoveConfigFile(UnitTestHelper.COMMANDLINE_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.COMMANDLINE_CONFIG);

            // specify namespace
            args = new string[] {"-n:TopCoder.File.Statistics.Main", "../../test_files/Processor"};
            CommandLine.Main(args);

            // specify base directory
            args = new string[] {"-b:../../test_files", "../../test_files/Processor"};
            CommandLine.Main(args);

            // recursively process sub-directory
            args = new string[] {"-r", "../../test_files/Processor"};
            CommandLine.Main(args);

            // specify output file
            // -o test_files/output.html a.txt b.java
            args = new string[] {"-o:../../test_files/output.html", "../../test_files/Processor"};
            CommandLine.Main(args);

            // restore the console output
            Console.SetOut(oldOut);
        }

        /// <summary>
        /// Create an instance of ExtendedFileStatistics.
        /// </summary>
        ///
        /// <returns>the create instance of ExtendedFileStatistics.</returns>
        private FileStatistics CreateStatistics()
        {
            // create a file statistics instance
            FileStatistics statistics = new ExtendedFileStatistics();

            // create file matchers
            IFileMatcher matcher1 = new AnyFileMatcher();
            IFileMatcher matcher2 = new ExtensionMatcher("txt");
            IFileMatcher matcher3 = new RegexMatcher(new Regex("pattern"));
            IFileMatcher matcher4 = new TextFileMatcher();
            IFileMatcher matcher5 = new BinaryFileMatcher();
            IFileMatcher matcher6 = new TypeNameMatcher("Java");
            IFileMatcher matcher7 = new MimeMatcher("text/plain");

            // create file processors
            IFileProcessor processor1 = new TextLineCounter("Text File", matcher1);
            IFileProcessor processor2 = new CStyleLineCounter("Source File", matcher6);

            // create stats reporting implementation
            IStatsReporting reporting1 = new BasicStatsReporting();

            // manipulate file processors
            // add file processor
            statistics.AddFileProcessor(processor1);
            statistics.AddFileProcessor(processor2);

            return statistics;
        }
    }
}