using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Unit test for abstract <see cref="MimeMatcher"/> class.
    /// static member: MimesPropertyKey
    /// constructor:TypeNameMatcher(string), TypeNameMatcher(IList), TypeNameMatcher(string, string)
    /// member: Match(FileType)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class MimeMatcherUnitTest
    {
        /// <summary>
        /// Represents an instance of <see cref="MimeMatcher"/>, used to test it's public member.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.MAGIC_NUMBER_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);

            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.INVALID_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.MAGIC_NUMBER_CONFIG);

            matcher = new MimeMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor4.matcher.");
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [Test]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.MAGIC_NUMBER_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
        }

        /// <summary>
        /// Test static member MimesPropertyKey, it should be 'mimes'.
        /// </summary>
        [Test]
        public void TestMimesPropertyKey()
        {
            Assert.AreEqual("mimes", MimeMatcher.MimesPropertyKey,
                "MimesPropertyKey of MimeMatcher should be 'mimes'.");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string mime), with mime is null, ANE is expected..
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_MimeIsNull()
        {
            new MimeMatcher((string) null);
        }

        /// <summary>
        /// Test ctor MimeMatcher(string mime), with mime is empty string, AE is expected..
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor1_MimeIsEmpty()
        {
            new MimeMatcher("  ");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string mime), with mime is non-empty string,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(new MimeMatcher("txt"),
                "Failed to create instance of MimeMatcher.");
        }


        /// <summary>
        /// Test ctor MimeMatcher(IList mimes), with mimes is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_MimesIsNull()
        {
            new TypeNameMatcher((IList) null);
        }

        /// <summary>
        /// Test ctor MimeMatcher(IList mimes), with mimes is empty list,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_MimesIsEmpty()
        {
            new TypeNameMatcher(new ArrayList());
        }

        /// <summary>
        /// Test ctor MimeMatcher(IList mimes), with mimes contains null,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_MimesContainsNull()
        {
            IList mimes = new ArrayList();
            mimes.Add(null);
            mimes.Add("txt");

            new TypeNameMatcher(mimes);
        }

        /// <summary>
        /// Test ctor MimeMatcher(IList mimes), with mimes contains emtpy string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_MimesContainsEmpty()
        {
            IList mimes = new ArrayList();
            mimes.Add("  ");
            mimes.Add("txt");

            new MimeMatcher(mimes);
        }

        /// <summary>
        /// Test ctor MimeMatcher(IList mimes), with mimes contains non-string type,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_MimesContainsNonString()
        {
            IList mimes = new ArrayList();
            mimes.Add(new object());
            mimes.Add("txt");

            new MimeMatcher(mimes);
        }

        /// <summary>
        /// Test ctor MimeMatcher(IList mimes), with mimes is string list,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            IList mimes = new ArrayList();
            mimes.Add(".c");
            mimes.Add("txt");

            Assert.IsNotNull(new MimeMatcher(mimes),
                "Failed to create instance of MimeMatcher");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string nameSpace, string prefix), with namespace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_NameSpaceIsNull()
        {
            new MimeMatcher(null, "processor4.matcher.");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string nameSpace, string prefix), with namespace is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor3_NameSpaceIsEmpty()
        {
            new MimeMatcher("  ", "processor4.matcher.");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string nameSpace, string prefix), with prefix is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_PrefixIsNull()
        {
            new MimeMatcher(UnitTestHelper.DEFAULT_NAMESPACE, null);
        }

        /// <summary>
        /// Test ctor MimeMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError1()
        {
            new MimeMatcher(UnitTestHelper.INVALID_NAMESPACE, "MimeMatcher1.");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError2()
        {
            new MimeMatcher(UnitTestHelper.INVALID_NAMESPACE, "MimeMatcher2.");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError3()
        {
            new MimeMatcher(UnitTestHelper.INVALID_NAMESPACE, "MimeMatcher3.");
        }

        /// <summary>
        /// Test ctor MimeMatcher(string nameSpace, string prefix), with both arguments are valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor3()
        {
            Assert.IsNotNull(matcher, "Failed to create instance of TypeNameMatcher.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file type not existed, false is expected.
        /// </summary>
        [Test]
        public void TestMatch_NotExisted()
        {
            Assert.IsFalse(matcher.Match(new FileInfo("../../test_files/matcher/test.txt")),
                "When file type is not existed, flase is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file mime contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch2()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.c")),
                "When file type is existed, true is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file mime contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch3()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.cpp")),
                "When file type is existed, true is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file mime contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch4()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.cs")),
                "When file type is existed, true is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file mime contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch5()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.java")),
                "When file type is existed, true is expected.");
        }
    }
}
