using System;
using System.IO;
using System.Collections;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Matcher
{
    /// <summary>
    /// Unit test for abstract <see cref="TypeNameMatcher"/> class.
    /// It mainly tests the following:
    /// static member: TypesPropertyKey
    /// constructor:TypeNameMatcher(string), TypeNameMatcher(IList), TypeNameMatcher(string, string)
    /// member: Match(FileType)
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class TypeNameMatcherUnitTest
    {
        /// <summary>
        /// Represents an instance of <see cref="TypeNameMatcher"/>, used to test it's public member.
        /// </summary>
        private IFileMatcher matcher;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.MAGIC_NUMBER_CONFIG);

            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.INVALID_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.MAGIC_NUMBER_CONFIG);

            matcher = new TypeNameMatcher(UnitTestHelper.DEFAULT_NAMESPACE, "processor2.matcher.");
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.MAGIC_NUMBER_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
        }

        /// <summary>
        /// Test static member TypesPropertyKey, it should be 'types'.
        /// </summary>
        [Test]
        public void TestTypesPropertyKey()
        {
            Assert.AreEqual("types", TypeNameMatcher.TypesPropertyKey,
                "TypesPropertyKey of TypeNameMatcher should be 'types'.");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string type), with type is null, ANE is expected..
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_TypeIsNull()
        {
            new TypeNameMatcher((string) null);
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string type), with type is empty string, AE is expected..
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor1_TypeIsEmpty()
        {
            new TypeNameMatcher("  ");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string type), with type is non-empty string,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(new TypeNameMatcher("txt"),
                "Failed to create instance of FileTypeMatcher.");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(IList types), with types is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_TypesIsNull()
        {
            new TypeNameMatcher((IList) null);
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(IList types), with types is empty list,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_TypesIsEmpty()
        {
            new TypeNameMatcher(new ArrayList());
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(IList types), with types contains null,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_TypesContainsNull()
        {
            IList types = new ArrayList();
            types.Add(null);
            types.Add("txt");

            new TypeNameMatcher(types);
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(IList types), with types contains emtpy string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_TypesContainsEmpty()
        {
            IList types = new ArrayList();
            types.Add("  ");
            types.Add("txt");

            new TypeNameMatcher(types);
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(IList types), with types contains non-string type,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_TypesContainsNonString()
        {
            IList types = new ArrayList();
            types.Add(new object());
            types.Add("txt");

            new TypeNameMatcher(types);
        }


        /// <summary>
        /// Test ctor TypeNameMatcher(IList types), with types is string list,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            IList types = new ArrayList();
            types.Add(".c");
            types.Add("txt");

            Assert.IsNotNull(new TypeNameMatcher(types),
                "Failed to create instance of TypeNameMatcher");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string nameSpace, string prefix), with namespace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_NameSpaceIsNull()
        {
            new TypeNameMatcher(null, "processor2.matcher.");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string nameSpace, string prefix), with namespace is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor3_NameSpaceIsEmpty()
        {
            new TypeNameMatcher("  ", "processor2.matcher.");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string nameSpace, string prefix), with prefix is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor3_PrefixIsNull()
        {
            new TypeNameMatcher(UnitTestHelper.DEFAULT_NAMESPACE, null);
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError1()
        {
            new TypeNameMatcher(UnitTestHelper.INVALID_NAMESPACE, "TypeNameMatcher1.");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError2()
        {
            new TypeNameMatcher(UnitTestHelper.INVALID_NAMESPACE, "TypeNameMatcher2.");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor3_ConfigFileError3()
        {
            new TypeNameMatcher(UnitTestHelper.INVALID_NAMESPACE, "TypeNameMatcher3.");
        }

        /// <summary>
        /// Test ctor TypeNameMatcher(string nameSpace, string prefix), with both arguments are valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor3()
        {
            Assert.IsNotNull(matcher, "Failed to create instance of TypeNameMatcher.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file type not existed, false is expected.
        /// </summary>
        [Test]
        public void TestMatch_NotExisted()
        {
            Assert.IsFalse(matcher.Match(new FileInfo("../../test_files/matcher/test.txt")),
                "When file type is not existed, flase is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file type contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch2()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.c")),
                "When file type is existed, true is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file type contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch3()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.cpp")),
                "When file type is existed, true is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file type contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch4()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.cs")),
                "When file type is existed, true is expected.");
        }

        /// <summary>
        /// Test Match(FileType type), since it is protected, test Match(FileInfo file) instead.
        /// with file type contained, true is expected.
        /// </summary>
        [Test]
        public void TestMatch5()
        {
            Assert.IsTrue(matcher.Match(new FileInfo("../../test_files/matcher/type.java")),
                "When file type is existed, true is expected.");
        }
    }
}
