using System;
using System.IO;
using NUnit.Framework;

using TopCoder.File.Statistics.Matcher;

namespace TopCoder.File.Statistics.Processor
{
    /// <summary>
    /// Unit test for <see cref="FileProcessorBase"/> class.
    /// It mainly tests the following:
    /// static member:AliasPropertyKey, FileMatcherPropertyKey, ClassnamePropertyKey
    /// constructor: FileProcessorBase(string, IFileMatcher), FileProcessorBase(string, string)
    /// property: Alias, FileMatcher
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class FileProcessorBaseUnitTest
    {
        /// <summary>
        /// Represents an instance of <see cref="IFileMatcher"/>, used to construct the processor.
        /// </summary>
        private IFileMatcher matcher = new AnyFileMatcher();

        /// <summary>
        /// Represents an instance of <see cref="FileProcessorBase"/>, used to test it's public member.
        /// </summary>
        private IFileProcessor processor;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.INVALID_CONFIG);

            processor = new MockFileProcessorBase("MockBase", matcher);
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.INVALID_CONFIG);
        }

        /// <summary>
        /// Test static member AliasPropertyKey, it should be 'alias'.
        /// </summary>
        [Test]
        public void TestAliasPropertyKey()
        {
            Assert.AreEqual("alias", FileProcessorBase.AliasPropertyKey,
                "FileProcessorBase of FileProcessorBase should be 'alias'");
        }

        /// <summary>
        /// Test static member FileMatcherPropertyKey, it should be 'matcher'.
        /// </summary>
        [Test]
        public void TestFileMatcherPropertyKey()
        {
            Assert.AreEqual("matcher", FileProcessorBase.FileMatcherPropertyKey,
                "FileProcessorBase of FileProcessorBase should be 'matcher'");
        }

        /// <summary>
        /// Test static member ClassnamePropertyKey, it should be 'classname'.
        /// </summary>
        [Test]
        public void TestClassnamePropertyKey()
        {
            Assert.AreEqual("classname", FileProcessorBase.ClassnamePropertyKey,
                "ClassnamePropertyKey of FileProcessorBase should be 'classname'");
        }

        /// <summary>
        /// Test property Alias Getter, it should return the ctor's argument or
        /// the alias load from config file.
        /// </summary>
        [Test]
        public void TestAlias_Getter()
        {
            Assert.AreEqual("MockBase", processor.Alias,
                "Failed to get the correct alias");
        }

        /// <summary>
        /// Test property Macther Getter, it should return the ctor's argument or
        /// the instance construct according the config file.
        /// </summary>
        [Test]
        public void TestMatcher_Getter()
        {
            Assert.AreEqual(matcher, processor.FileMatcher,
                "Failed to get the correct File Matcher");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string alias, IFileMatcher matcher), with alias is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_AliasIsNull()
        {
            new MockFileProcessorBase(null, matcher);
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string alias, IFileMatcher matcher), with alias is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor1_AliasIsEmpty()
        {
            new MockFileProcessorBase("  ", matcher);
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string alias, IFileMatcher matcher), with matcher is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor1_MatcherIsNull()
        {
            new MockFileProcessorBase("MockBase", (IFileMatcher) null);
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string alias, IFileMatcher matcher), with both arguments are valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(matcher, "Failed to create instance of FileProcessorBase.");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_NamespaceIsNull()
        {
            new MockFileProcessorBase(null, "processor1");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with nameSpace is emtpy string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_NamespaceIsEmpty()
        {
            new MockFileProcessorBase("  ", "processor1");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with prefix is null.
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_PrefixIsNull()
        {
            new MockFileProcessorBase(UnitTestHelper.DEFAULT_NAMESPACE, (string) null);
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor1_ConfigFileError1()
        {
            new MockFileProcessorBase(UnitTestHelper.INVALID_NAMESPACE, "Processor1");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor2_ConfigFileError2()
        {
            new MockFileProcessorBase(UnitTestHelper.INVALID_NAMESPACE, "Processor2");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor2_ConfigFileError3()
        {
            new MockFileProcessorBase(UnitTestHelper.INVALID_NAMESPACE, "Processor3");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with config file not correct,
        /// ConfigurationException is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ConfigurationException))]
        public void TestCtor2_ConfigFileError4()
        {
            new MockFileProcessorBase(UnitTestHelper.INVALID_NAMESPACE, "Processor4");
        }

        /// <summary>
        /// Test ctor FileProcessorBase(string nameSpace, string prefix), with both argument are valid,
        /// instance should be created.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            processor = new MockFileProcessorBase(UnitTestHelper.DEFAULT_NAMESPACE, "processor1");
            Assert.IsNotNull(processor, "Failed to create instance of FileProcessorBase.");
        }
    }
}