using System;
using System.IO;

using TopCoder.File.Statistics.Matcher;
using TopCoder.File.Statistics.Processor.LineCounter;
using NUnit.Framework;

namespace TopCoder.File.Statistics.Reporting
{
    /// <summary>
    /// Unit Test for <see cref="BasicStatsReporting"/> class.
    /// It mainly tests the following:
    /// constructor: BasicStatsReporting(), BasicStatsReporting(string, string)
    /// member: GenerateReport()
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (C) 2005 TopCoder Inc., All Rights Reserved.</copyright>
    /// </summary>
    [TestFixture]
    public class BasicStatsReportingUnitTest
    {
        /// <summary>
        /// Represent the instance of <see cref="IFileProcessor"/> implementation,
        /// used to construct the report.
        /// </summary>
        private IFileProcessor processor = new TextLineCounter("Text", new AnyFileMatcher());

        /// <summary>
        /// Represents the instance of FileInfo, used to construct the report.
        /// </summary>
        private FileInfo file = new FileInfo("../../test_files/processor/lineCounter/text1.txt");

        /// <summary>
        /// Represents the instance of <see cref="StatsResult"/>, used to construct the report.
        /// </summary>
        private StatsResult result = null;

        /// <summary>
        /// Represents an instance of <see cref="BasicStatsReporting"/>, used to test it public member.
        /// </summary>
        private BasicStatsReporting report;

        /// <summary>
        /// Set up for each test.
        /// </summary>
        [SetUp]
        public void RunBeforeEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
            UnitTestHelper.LoadConfigFile(UnitTestHelper.DEFAULT_CONFIG);

            result = new StatsResult();
            Metric metric = new Metric(TextLineCounter.Name, TextLineCounter.DisplayName, TextLineCounter.Name);
            result[metric] = 1;

            report = new BasicStatsReporting(UnitTestHelper.DEFAULT_NAMESPACE, "reporting.");
        }

        /// <summary>
        /// Tear down for each test.
        /// </summary>
        [TearDown]
        public void RunAfterEachTest()
        {
            UnitTestHelper.RemoveConfigFile(UnitTestHelper.DEFAULT_CONFIG);
        }

        /// <summary>
        /// Test ctor BasicStatsReporting(), instance should be created.
        /// </summary>
        [Test]
        public void TestCtor1()
        {
            Assert.IsNotNull(new BasicStatsReporting(), "Failed to create instance of BasicStatsReporting.");
        }

        /// <summary>
        /// Test ctor BasicStatsReporting(string nameSpace, string prefix), with nameSpace is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_NamespaceIsNull()
        {
            new BasicStatsReporting(null, "reporting.");
        }

        /// <summary>
        /// Test ctor BasicStatsReporting(string nameSpace, string prefix), with nameSpace is empty string,
        /// AE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentException))]
        public void TestCtor2_NamespaceIsEmpty()
        {
            new BasicStatsReporting("  ", "reporting.");
        }

        /// <summary>
        /// Test ctor BasicStatsReporting(string nameSpace, string prefix), with prefix is null,
        /// ANE is expected.
        /// </summary>
        [Test, ExpectedException(typeof(ArgumentNullException))]
        public void TestCtor2_PrefixIsNull()
        {
            new BasicStatsReporting("topcoder", null);
        }

        /// <summary>
        /// Test ctor BasicStatsReporting(string nameSpace, string prefix), with nameSpace is empty string,
        /// AE is expected.
        /// </summary>
        [Test]
        public void TestCtor2()
        {
            Assert.IsNotNull(report, "Failed to create instance of BasicStatsReporting.");
        }

        /// <summary>
        /// Test GenerateReport(), aggregate some result and generte the report,
        /// and verify the report.
        /// </summary>
        [Test]
        public void TestGenerateReport1()
        {
            report.Reset(new DirectoryInfo("../../test_files"));

            // process file
            FileInfo file = new FileInfo("../../test_files/processor/lineCounter/text1.txt");
            processor = new TextLineCounter("Text", new AnyFileMatcher());

            // aggregate one file
            report.Aggregate(processor, file, processor.Process(file));

            string content = report.GenerateReport();
            UnitTestHelper.VerifyResult(content, "../../test_files/Reporting/basic1.txt",
                "The generated report is not expected.");
        }

        /// <summary>
        /// Test GenerateReport(), aggregate some result and generte the report,
        /// and verify the report.
        /// </summary>
        [Test]
        public void TestGenerateReport2()
        {
            report.Reset(new DirectoryInfo("../../test_files"));
            FileInfo file1 = new FileInfo("../../test_files/processor/lineCounter/text1.txt");
            FileInfo file2 = new FileInfo("../../test_files/processor/lineCounter/text2.txt");
            FileInfo file3 = new FileInfo("../../test_files/processor/lineCounter/text3.txt");
            FileInfo file4 = new FileInfo("../../test_files/processor/lineCounter/text4.txt");
            FileInfo file5 = new FileInfo("../../test_files/processor/lineCounter/text5.txt");
            FileInfo file6 = new FileInfo("../../test_files/processor/lineCounter/sub/text4.txt");
            FileInfo file7 = new FileInfo("../../test_files/processor/lineCounter/sub/text5.txt");

            processor = new TextLineCounter("Text", new AnyFileMatcher());

            // aggregate files
            report.Aggregate(processor, file1, processor.Process(file1));
            report.Aggregate(processor, file2, processor.Process(file2));
            report.Aggregate(processor, file3, processor.Process(file3));
            report.Aggregate(processor, file4, processor.Process(file4));
            report.Aggregate(processor, file5, processor.Process(file5));
            report.Aggregate(processor, file6, processor.Process(file6));
            report.Aggregate(processor, file7, processor.Process(file7));

            string content = report.GenerateReport();
            UnitTestHelper.VerifyResult(content, "../../test_files/Reporting/basic2.txt",
                "The generated report is not expected.");
        }

        /// <summary>
        /// Test GenerateReport(), aggregate some result and generte the report,
        /// and verify the report.
        /// </summary>
        [Test]
        public void TestGenerateReport3()
        {
            report.Reset(new DirectoryInfo("../../test_files"));

            // process file
            FileInfo file1 = new FileInfo("../../test_files/processor/lineCounter/text3.txt");
            FileInfo file2 = new FileInfo("../../test_files/processor/lineCounter/text10.c");
            processor = new TextLineCounter("Text", new AnyFileMatcher());
            IFileProcessor processor2 = new TextLineCounter("c", new ExtensionMatcher("c"));

            // aggregate one file
            report.Aggregate(processor, file1, processor.Process(file1));
            report.Aggregate(processor2, file2, processor2.Process(file2));

            string content = report.GenerateReport();
            UnitTestHelper.VerifyResult(content, "../../test_files/Reporting/basic3.txt",
                "The generated report is not expected.");
        }

        /// <summary>
        /// Test GenerateReport(), aggregate some result and generte the report,
        /// and verify the report.
        /// </summary>
        [Test]
        public void TestGenerateReport4()
        {
            report.Reset(new DirectoryInfo("../../test_files"));

            // process file
            FileInfo file1 = new FileInfo("../../test_files/processor/lineCounter/text3.txt");
            FileInfo file2 = new FileInfo("../../test_files/processor/lineCounter/text10.c");
            processor = new TextLineCounter("Text", new AnyFileMatcher());
            IFileProcessor processor2 = new TextLineCounter("c", new ExtensionMatcher("c"));

            // aggregate one file
            report.Aggregate(processor, file1, processor.Process(file1));
            report.Aggregate(processor2, file2, processor2.Process(file2));

            string content = report.GenerateReport();
            UnitTestHelper.VerifyResult(content, "../../test_files/Reporting/basic4.txt",
                "The generated report is not expected.");
        }
    }
}
